<?php
/**
 * This file is part of 
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 1.1.2.3 $
 * $Author: tora60 $
 * $Date: 2007/12/22 14:11:51 $
 *
 *
 * Copyright (c) 2003 Brian E. Lozier (brian@massassi.net)
 *
 * set_vars() method contributed by Ricardo Garcia (Thanks!)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

defined('mxMainFileLoaded') or die('access denied');

class PHP_Template {
    var $vars; /// Holds all the template variables
    var $path; /// Path to the templates

    /**
     * Constructor
     *
     * @param string $path the path to the templates
     * @return void
     */
    function PHP_Template($path = null)
    {
        $this->path = $path;
        $this->vars = array();
    }

    /**
     * Set the path to the template files.
     *
     * @param string $path path to template files
     * @return void
     */
    function set_path($path)
    {
        $this->path = $path;
    }

    /**
     * Set a template variable.
     *
     * @param string $name name of the variable to set
     * @param mixed $value the value of the variable
     * @return void
     */
    function set($name, $value)
    {
        $this->vars[$name] = $value;
    }

    /**
     * Set a bunch of variables at once using an associative array.
     *
     * @param array $vars array of vars to set
     * @param bool $clear whether to completely overwrite the existing vars
     * @return void
     */
    function set_vars($vars, $clear = false)
    {
        if ($clear) {
            $this->vars = $vars;
        } else {
            if (is_array($vars)) $this->vars = array_merge($this->vars, $vars);
        }
    }

    /**
     * Open, parse, and return the template file.
     *
     * @param string $ string the template file name
     * @return string
     */
    function fetch($file)
    {
        extract($this->vars); // Extract the vars to local namespace
        ob_start(); // Start output buffering
        include($this->path . $file); // Include the file
        $contents = ob_get_contents(); // Get the contents of the buffer
        ob_end_clean(); // End buffering and discard
        return $contents; // Return the contents
    }
} # endofclass
class Mx_Template extends PHP_Template {
    var $cache_id;
    var $expire;
    var $cached;
    var $container;

    function Mx_Template($path = null, $cache_id = null, $expire = 900)
    {
        if (!is_dir($path)) {
            include("header.php");
            print('<h3>No template directory found...</h3>');
            include("footer.php");
            die();
        }
        parent::PHP_Template($path);
        $this->cache_id = $cache_id ? CACHE_DIR . md5($cache_id) : $cache_id;
        $this->expire = $expire;
    }

    function fetch($file, $section = '')
    {
        if (!file_exists($this->path . $file)) {
            return '';
        } else {
            extract($this->vars); // Extract the vars to local namespace
            ob_start(); // Start output buffering
            include($this->path . $file); // Include the file
            $contents = ob_get_contents(); // Get the contents of the buffer
            ob_end_clean();
            return $contents; // Return the contents
        }
    }

    function assign_void($name, $fnc)
    {
        ob_start();
        eval($fnc . ";");
        $buf = ob_get_contents();
        ob_end_clean();
        parent::set($name, $buf);
    }

    function display($file, $section = '')
    {
        print($this->fetch($file, $section));
    }

    function assign($name, $value = "")
    {
        if (empty($value) && is_array($name)) {
            parent::set_vars($name);
        } else {
            parent::set($name, $value);
        }
    }

    function is_cached()
    {
        if ($this->cached) return true;
        // Passed a cache_id?
        if (!$this->cache_id) return false;
        // Cache file exists?
        if (!file_exists($this->cache_id)) return false;
        // Can get the time of the file?
        if (!($mtime = filemtime($this->cache_id))) return false;
        // Cache expired?
        if (($mtime + $this->expire) < time()) {
            @unlink($this->cache_id);
            return false;
        } else {
            /**
             * Cache the results of this is_cached() call.  Why?  So
             * we don't have to double the overhead for each template.
             * If we didn't cache, it would be hitting the file system
             * twice as much (file_exists() & filemtime() [twice each]).
             */
            $this->cached = true;
            return true;
        }
    }

    function fetch_cache($file)
    {
        if ($this->is_cached()) {
            $fp = @fopen($this->cache_id, 'r');
            $contents = fread($fp, filesize($this->cache_id));
            fclose($fp);
            return $contents;
        } else {
            $contents = $this->fetch($file);
            // Write the cache
            if ($fp = @fopen($this->cache_id, 'w')) {
                fwrite($fp, $contents);
                fclose($fp);
            } else {
                die('Unable to write cache.');
            }
            return $contents;
        }
    }

    function display_cache($file)
    {
        print ($this->fetch_cache($file));
    }
} # endofclass

?>