<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 1.27.2.15 $
 * $Author: tora60 $
 * $Date: 2008/01/29 21:20:14 $
 */

defined('mxMainFileLoaded') or die('access denied');

/**
 * Bindet das aktuelle Sprachfile ein
 *
 * @param string $module Name des Modules fuer das das Sprachfile
 * eingebunden werde soll
 * @return Gibt false oder den Namen des aktuellen language-Files zurueck
 */
function mxGetLangfile($module)
{
    if (preg_match("#(:)|(\.\.)|(\/)|(\\\)#", $module)) {
        mxYoubad('Bad Request for Modulename');
        die();
    }
    $lang = $GLOBALS['currentlang'];
    $langpath = PMX_REAL_BASE_DIR . '/' . (($module == 'admin') ? 'admin/language' : 'modules/' . $module . '/language');
    $ok = @include_once($langpath . '/lang-' . $lang . '.php');
    if ($ok) return $lang;
    // deutsch_du noch beachten und ggf. includen
    if (strpos($lang, 'german')) {
        $xlang = ($lang == 'german') ? 'german_du' : 'german';
        $ok = @include_once($langpath . '/lang-' . $xlang . '.php');
        if ($ok) return $xlang;
    }
    // / Admin erhaelt auf jeden Fall eine Meldung
    if (MX_IS_ADMIN) echo mxSiteServiceMessage("Error: language-file '" . $lang . "' for module '" . $module . "' is missing!");
    $handle = @opendir($langpath);
    if ($handle) {
        while (false !== ($file = @readdir($handle))) {
            if (preg_match("/^lang\-(.+)\.php/i", $file)) {
                $ok = @include_once($langpath . '/' . $file); /// alle verfuegbaren includen
                // break;                                   /// um dies zu vermeiden, diese Zeile einkommentieren
            }
        }
        closedir($handle);
    }
    // / Meldung fuer User nur ausgeben, wenn keine Sprachdatei includet
    if (!MX_IS_ADMIN && empty($ok)) echo mxSiteServiceMessage("Error: language-file '" . $lang . "' for module '" . $module . "' is missing!");
    return false;
}

/**
 * Setzen einer Session Variablen
 *
 * @param string $name Name der Sesision-Variablen die gesetzt werden soll
 * @param string $value Wert der Session-variablen der gesetzt werden soll
 * @return bool
 */
function mxSessionSetVar($name, $value)
{
    $varname = MX_SESSION_VARPREFIX . $name;
    $_SESSION[$varname] = $value;
    return true;
}

/**
 * Auslesen einer Session-variablen
 *
 * @param string $name Name der Sesision-Variablen der ausgeselen werden soll
 * @return mixed Gibt der Wert der Session Variablen zurueck
 */
function mxSessionGetVar($name)
{
    $varname = MX_SESSION_VARPREFIX . $name;
    if (empty($_SESSION[$varname])) return false;
    return $_SESSION[$varname];
}

/**
 * Loeschen einser Session-variablen
 *
 * @param string $name Name der Session-Variablen, der geloescht werden soll.
 * @return bool
 */
function mxSessionDelVar($name)
{
    $varname = MX_SESSION_VARPREFIX . $name;
    unset($_SESSION[$varname]);
    session_unregister($varname);
    return true;
}

/**
 * Zerstoeren der aktuellen Session
 *
 * @return bool
 */
function mxSessionDestroy()
{
    mxSetCookie(MX_SESSION_NAME, "", -1);
    mxSetCookie(MX_SAFECOOKIE_NAME_USER, "", -1);
    mxSetCookie(MX_SAFECOOKIE_NAME_ADMIN, "", -1);
    mxSetNukeCookie("user");
    mxSetNukeCookie("admin");
    foreach($_SESSION as $varname => $value) {
        unset($_SESSION[$varname]);
    }
    session_destroy();
    return true;
}

/**
 * Reinigen User-Input / erwzingen double oder integer
 * Nimmt eine globale Variable und modifiziert den Inhalt erzwingt double bzw. integer
 *
 * @param mixed $var Name der Variablen
 * @return mixed Liefert die modifizierte Variable zurueck, wenn nur eine Variable uebergeben wurde,
 * ansonsten ein Array mit den modifizierten Werten
 */
function mxForceInteger()
{
    $arr = func_get_args();
    foreach($arr as $i => $v) {
        if (empty($arr[$i])) {
            $arr[$i] = 0;
            continue;
        }
        if (is_array($arr[$i])) {
            foreach($arr[$i] as $ia => $va) {
                if (empty($arr[$i][$ia])) {
                    $arr[$i][$ia] = 0;
                    continue;
                }
                $arr[$i][$ia] = mxForceInteger($arr[$i][$ia]);
            }
        } else {
            $arr[$i] = (gettype($arr[$i]) == "double") ? (double)$arr[$i] : (int)$arr[$i];
        }
    }
    $out = (func_num_args() == 1) ? $arr[0] : $arr;
    return $out;
}

/**
 * Modifiziert Variableninhalt zum Speichern in der Datenbank (Escapen von Zeichen)
 * Umkehrfunktion von mxStripSlashes()
 *
 * Nimmt eine Variable und modifiziert den Inhalt so das dieser
 * ohne Probleme in der Datenbank gespeichert werden kann
 *
 * @param mixed $what Name der variable deren Inhalt modifiziert werden soll
 * @return mixed Liefert die modifizierte Variable zurueck, wenn nur eine Variable uebergeben wurde,
 * ansonsten ein Array mit den modifizierten Werten
 */
function mxAddSlashesForSQL($what)
{
    // wenn $what numerisch oder leer ist, unveraendert zurueckgeben
    if (is_numeric($what) || empty($what)) return $what;
    static $entity;
    if (empty($entity)) {
        // eindeutige Kennung erstellen
        mt_srand((double)microtime() * 1000000);
        $entity = md5(mt_rand());
    }
    // wenn $what ein Array, das Array durchlaufen
    // und die Funktion rekursiv aufrufen
    if (is_array($what)) {
        foreach ($what as $key => $value) {
            $what[$key] = mxAddSlashesForSQL($value);
        }
    }
    // wenn String
    else {
        // erwuenschte Backslashes in eindeutigen String umwandeln
        $what = str_replace("\\\\", $entity, $what);
        // alle anderen Backslashes entfernen
        $what = stripslashes($what);
        // erwuenschte Backslashes wieder zurueckwandeln
        $what = str_replace($entity, "\\", $what);
        // sicherstellen, dass die richtige Anzahl von Backslashes wieder eingefuegt werden
        $what = addslashes($what);
    }
    return $what;
}

/**
 * Modifiziert Datenbankausgabe (Escapen von Zeichen rueckgaengig machen
 * Umkehrfunktion von mxAddSlashesForSQL()
 * Nimmt eine Variable und modifiziert den Inhalt so das dieser
 * ohne Probleme angezeigt werden kann
 *
 * @param mixed $what Name der Variablen deren Inhalt modifiziert werden soll
 * @return mixed Liefert die modifizierte Variable zurueck, wenn nur eine Variable uebergeben wurde,
 * ansonsten ein Array mit den modifizierten Werten
 */
function mxStripSlashes(&$what)
{
    // wenn $what leer ist, Leerstring zurueckgeben
    if (empty($what)) return "";
    // eindeutige Kennung erstellen
    static $entity;
    if (empty($entity)) {
        mt_srand((double)microtime() * 1000000);
        $entity = md5(mt_rand());
    }
    if (is_numeric($what) || empty($what)) {
        // numerische und leere Werte nicht behandeln
        return $what;
    } else if (is_array($what)) {
        // wenn Array, rekursiv aufrufen
        array_walk($what, 'mxStripSlashes');
    } else if (is_string($what)) {
        // nur wenn String...
        // erwuenschte Backslashes in eindeutigen String umwandeln
        $what = str_replace("\\\\", $entity, $what);
        // alle anderen Backslashes entfernen
        $what = stripslashes($what);
        // erwuenschte Backslashes wieder zurueckwandeln
        $what = str_replace($entity, "\\", $what);
    }
    return $what;
}

/**
 * Konvertiere ein Newline (\n) in ein HTML <br> - Tag
 * Umkehrfunktion von mxUndoNltobr()
 *
 * @param string $value Name der Variablen deren Inhalt konvertiert werden soll
 * @return string Liefert konvertierten String zurueck
 */
function mxNL2BR($value)
{
    if (strip_tags($value) === $value) {
        return nl2br($value);
    }
    return $value;
}

/**
 * Konvertiere eine HTML <br> - Tag in ein newline (\n)
 * Umkehrfunktion von mxNL2BR()
 *
 * @param string $text Name der Variablen deren Inhalt konvertiert werden soll
 * @return string Liefert konvertierten String zurueck
 */
function mxUndoNltobr($text)
{
    return (preg_replace('/(<br[ \/]*?>)/i', "", $text));
}

/**
 * Rueckgaengigmachen des SQL-Injection Fixes um die Originalzeichen zurueckzuerhalten
 *
 * @param string $value Name der Variablen deren Inhalt konvertiert werden soll.
 * @return string Liefert konvertierten String zurueck
 */
function mxUndoSqlInjectFix(&$value)
{
    if (!empty($value)) {
        if (!is_long($value) && !is_int($value) && !is_double($value) && !is_object($value) && !is_array($value)) {
            $value = str_replace(array('&#41;', '&#124;', '&#45;&#45;', '&#35;'), array(')', '|', '--', '#'), $value);
            $value = str_replace('&amp;#', '&#xxxxxxxx', $value);
        }
    }
    return $value;
}

/**
 * Variablenwert fuer Ausgabe aufbereiten / Schutz vor E-mail sammlern
 * Nimmt einen Variablenwert und modifiziert den enthaltene Emailadresse so, das diese von
 * Email-Sammlern nicht verwendet werden koennen
 *
 * @param mixed $var variable/n deren Wert modifiziert werden soll
 * @return mixed Liefert die modifizierte Variable zurueck, wenn nur eine Variable uebergeben wurde,
 * ansonsten ein Array mit den modifizierten Werten
 */
function mxPrepareToDisplay()
{
    // This search and replace finds the text 'x@y' and replaces
    // it with HTML entities, this provides protection against
    // email harvesters
    $search = array('/(.)@(.)/se');
    $replace = array('"&#" .sprintf("%03d", ord("\\1")) .";&#064;&#" .sprintf("%03d", ord("\\2")) . ";";');
    $resarray = array();
    foreach (func_get_args() as $var) {
        $resarray[] = preg_replace($search, $replace, $var); // Add to array
    }
    if (func_num_args() == 1) { // Return vars
        return $resarray[0];
    } else {
        return $resarray;
    }
}

/**
 * Liefert ein Array mit den erlaubten HTML-Tags oder ein
 * Array mit PCRE um diese in weiteren Berarbeitungsschritten (preg_match etc) verwenden zu koennen
 *
 * @staticvar mixed $allowedhtmlpreg Statische Variable um die PCRE zu speichern
 * @staticvar mixed $allowedtags Statische Variable um die HTML-Tags zu speichern
 * @param int $format Variable um zwischen Rueckgabe PCRE oder HTML-Tags umzuschalten
 * @return mixed Liefert Array mit HTML-Tags oder Array mit PCRE zurueck
 */
function mxGetAllowedHtml($format = '')
{
    static $allowedhtmlpreg;
    static $allowedtags;
    if (!isset($allowedhtmlpreg)) {
        $allowedhtmlpreg = array();
        foreach($GLOBALS['AllowableHTML'] as $tag => $para) {
            switch ($para) {
                case 0:
                    continue;
                    break;
                case 1:
                    $allowedhtmlpreg[] = "|<(/?$tag)\s*/?>|i";
                    $allowedtags[] = $tag;
                    break;
                case 2:
                    $allowedhtmlpreg[] = "|<(/?$tag(\s+.*?)?)>|i";
                    $allowedtags[] = $tag;
                    break;
            }
        }
    }
    if ($format) {
        return $allowedhtmlpreg;
    }
    return $allowedtags;
}

/**
 * HTML fuer Ausgabe vorbereiten
 *
 * @deprecated Funktion seit pragmaMx 0.16 nicht mehr verwendet.
 * macht aus Kompatibilitaetsgruenden dasselbe wie mxPrepareToDisplay()
 * @param mixed $var variable/n deren Wert modifiziert werden soll
 * @return mixed Liefert die modifizierte Variable zurueck, wenn nur eine Variable uebergeben wurde,
 * ansonsten ein Array mit den modifizierten Werten
 */
function mxPrepareToHTMLDisplay()
{
    // / diese Funktion wird seit 0.1.6 nicht mehr ben?tigt
    // / zur Abwaertskompatibilitaet macht sie jetzt nur noch
    // / das gleiche wie mxPrepareToDisplay()
    // This search and replace finds the text 'x@y' and replaces
    // it with HTML entities, this provides protection against
    // email harvesters
    $search = array('/(.)@(.)/se');
    $replace = array('"&#" .sprintf("%03d", ord("\\1")) .";&#064;&#" .sprintf("%03d", ord("\\2")) . ";";');
    $resarray = array();
    foreach (func_get_args() as $var) {
        // Add to array
        $resarray[] = preg_replace($search, $replace, $var);
    }
    if (func_num_args() == 1) { // Return vars
        return $resarray[0];
    } else {
        return $resarray;
    }
}

/**
 * Zensierte Woerter entfernen
 *
 * @param mixed $var Variable deren Inhalt Zensiert werden soll
 * @return mixed Liefert die modifizierte Variable zurueck, wenn nur eine Variable uebergeben wurde,
 * ansonsten ein Array mit den zensierten Werten
 */
function mxPrepareCensored()
{
    static $docensor;
    if (!isset($docensor)) {
        $docensor = $GLOBALS['CensorMode'];
    }
    static $search = array();
    if (empty($search)) {
        $repsearch = array('/o/i', '/e/i', '/a/i', '/i/i');
        $repreplace = array('0', '3', '@', '1');
        $badwords = $GLOBALS['CensorList'];
        foreach ($badwords as $badword) {
            $search[] = "#\b" . preg_quote($badword, '#') . "\b#i"; // Simple word
            $mungedword = preg_replace($repsearch, $repreplace, $badword); // Common replacements
            if ($mungedword != $badword) {
                $search[] = "#\b" . preg_quote($mungedword, '#') . "\b#i";
            }
        }
    }
    $replace = $GLOBALS['CensorReplace'];
    $resarray = array();
    foreach (func_get_args() as $var) {
        if ($docensor) {
            $var = @preg_replace($search, $replace, $var); // Parse out nasty words
        }
        $resarray[] = $var; // Add to array
    }
    if (func_num_args() == 1) { // Return vars
        return $resarray[0];
    } else {
        return $resarray;
    }
}

/**
 * Namen des verwendeten Themes abfragen
 *
 * @static string $usertheme Statische Variable um den Namen des verwendeten Themes zu speichern
 * @return returns Liefert den namen des verwendeten Themes zurueck
 */
function mxGetTheme()
{
    static $usertheme;
    if (isset($usertheme)) {
        return $usertheme;
    }
    if (MX_IS_USER) {
        $usersession = mxGetUserSession();
        $usertheme = (empty($usersession[9])) ? $GLOBALS['Default_Theme'] : $usersession[9];
        $usertheme = (!empty($_COOKIE['themetest'])) ? $_COOKIE['themetest'] : $usertheme; // evtl. themeaenderung?
        $usertheme = (!empty($_REQUEST['theme'])) ? $_REQUEST['theme'] : $usertheme; // evtl. themeaenderung?
    } else {
        $usertheme = $GLOBALS['Default_Theme'];
        $usertheme = (!empty($_COOKIE['themetest'])) ? $_COOKIE['themetest'] : $usertheme; // evtl. themeaenderung?
    }
    if (!@is_file(PMX_REAL_BASE_DIR . '/themes/' . $usertheme . '/theme.php')) {
        $usertheme = $GLOBALS['Default_Theme'];
    }
    if (!@is_file(PMX_REAL_BASE_DIR . '/themes/' . $usertheme . '/theme.php')) {
        $handle = @opendir(PMX_REAL_BASE_DIR . '/themes');
        if (!$handle) die('<h1>Error: the theme-folder is missing!</h1>');
        while (false !== ($file = readdir($handle))) {
            if (!ereg("\.", $file)) {
                if (@is_file(PMX_REAL_BASE_DIR . '/themes/' . $file . '/theme.php')) {
                    $usertheme = $file;
                    break;
                }
            }
        }
        closedir($handle);
        if (MX_IS_ADMIN) echo mxSiteServiceMessage("Error: the default-theme '" . $GLOBALS["Default_Theme"] . "' is missing!<br />[&nbsp;<a href=\"admin.php?op=Configure\">edit</a>&nbsp;]");
    }
    // versch. Themekonstanten noch checken..
    // if (!defined('MX_THEME_CACHABLE')) {
    // define('MX_THEME_CACHABLE', false);
    // }
    return($usertheme);
}

/**
 * erzeugt ein verstecktes Feld um die Gueltigkeit der Session
 * beim Userlogin anzuzeigen, zusaetzlich die benoetigten Hidden-Felder
 * Diese Funktion muss in Themes eingesetzt werden, die ein Userlogin haben
 * Aufruf: echo mxGetUserLoginCheckField();
 *
 * @return string Liefert die benoetigten Felder fuer ein HTML-Formular zurueck
 */
function mxGetUserLoginCheckField()
{
    mt_srand((double)microtime() * 1000000);
    // if, falls bereits im YA-Modul oder Loginblock definiert
    if (!mxSessionGetVar("uloginreqcheck")) mxSessionSetVar("uloginreqcheck", mt_rand());
    $out = "<input type=\"hidden\" name=\"check\" value=\"" . MD5(mxSessionGetVar("uloginreqcheck")) . "\" />\n"
     . "<input type=\"hidden\" name=\"check_cookie\" value=\"" . MD5(session_id()) . "\" />\n"
     . "<input type=\"hidden\" name=\"name\" value=\"Your_Account\" />\n"
     . "<input type=\"hidden\" name=\"op\" value=\"login\" />\n";
    return $out;
}

/**
 * ueberprueft, ob der Nutzer als Admin eingeloggt ist
 *
 * @static bool $isadmin Statische Variable um den Status zu speichern
 * @return boolean
 */
function mxIsAdmin()
{
    global $prefix;
    static $isadmin;
    if (empty($isadmin)) {
        if (!empty($GLOBALS["vkpSafeCookie2"])) { // nur wenn Variable gesetzt
            $checkId = md5(mxSessionGetVar("ck2"));
            // $cookiename = MX_SAFECOOKIE_NAME_ADMIN;
            $cookiechk = (empty($_COOKIE[MX_SAFECOOKIE_NAME_ADMIN])) ? "" : $_COOKIE[MX_SAFECOOKIE_NAME_ADMIN];
            // print "$cookiechk = $checkId";
            if ($cookiechk != $checkId) {
                $isadmin = -1;
                return 0;
            }
        }
        $theadmin = mxSessionGetVar("admin");
        if (empty($theadmin)) {
            $isadmin = -1;
            return 0;
        }
        $admin0 = base64_decode($theadmin);
        $admin1 = explode(":", $admin0);
        if (empty($admin1[1])) {
            $isadmin = -1;
            return 0;
        }
        $GLOBALS['mxSkipSqlDetect'] = true;
        $result = sql_query("select pwd from ${prefix}_authors where aid='" . mxAddSlashesForSQL(substr($admin1[0], 0, 25)) . "'");
        if (!$result) {
            $msg = str_replace($GLOBALS['prefix'], '{prefix}', strtolower("<div style=\"background-color: Red; color: Yellow; border: thin solid Black;\"><h1>Error:</h1><p>" . mysql_error() . "</p></div>"));
            $msg = str_replace(strtolower($GLOBALS['dbname']), '{dbname}', $msg);
            if (function_exists('mxpro_dberror')) {
                mxpro_dberror($msg);
            } else {
                echo $msg;
            }
            return false;
        }
        list($pass) = sql_fetch_row($result);
        sql_free_result($result);
        if ($pass == $admin1[1] && !empty($pass)) {
            $isadmin = 1;
            return 1;
        }
    } else {
        return ($isadmin == 1) ? 1 : 0;
    }
}

/**
 * Ueberprueft, ob der Nutzer als Admin eingeloggt ist
 *
 * @static bool $isuser Statische Variable um den Status zu speichern
 * @return boolean
 */
function mxIsUser()
{
    global $user_prefix;
    static $isuser;
    if (empty($isuser)) {
        if (!empty($GLOBALS["vkpSafeCookie1"])) { // nur wenn Variable gesetzt
            $checkId = md5(mxSessionGetVar("ck1"));
            $cookiechk = (empty($_COOKIE[MX_SAFECOOKIE_NAME_USER])) ? "" : $_COOKIE[MX_SAFECOOKIE_NAME_USER];
            // print "$cookiechk = $checkId";
            if ($cookiechk != $checkId) {
                $isuser = -1;
                return 0;
            }
        }
        $user0 = base64_decode(mxSessionGetVar("user"));
        $user1 = explode(":", $user0);
        if (empty($user1[2]) || !is_numeric($user1[0])) {
            $isuser = -1;
        } else {
            $GLOBALS['mxSkipSqlDetect'] = true;
            $result = sql_query("select uid, pass from ${user_prefix}_users where uid=" . intval($user1[0]) . " and uname='" . mxAddSlashesForSQL(substr($user1[1], 0, 25)) . "' and user_stat=1");
            if (!$result) {
                $msg = str_replace($GLOBALS['user_prefix'], '{user_prefix}', strtolower("<div style=\"background-color: Red; color: Yellow; border: thin solid Black;\"><h1>Error:</h1><p>" . mysql_error() . "</p></div>"));
                $msg = str_replace(strtolower($GLOBALS['dbname']), '{dbname}', $msg);
                if (function_exists('mxpro_dberror')) {
                    mxpro_dberror($msg);
                } else {
                    echo $msg;
                }
                return false;
            }
            list($uid, $pass) = sql_fetch_row($result);
            $isuser = (empty($uid) || ($pass != $user1[2])) ? -1 : 1;
            sql_free_result($result);
        }
        // return ($isuser == 1) ? mxSessionGetVar("userok") : 0;
        return ($isuser == 1) ? 1 : 0;
    } else {
        return ($isuser == 1) ? 1 : 0;
    }
}

/**
 * Abfragen der User-Session
 *
 * @static mixed $stat_cookie Variable zum Speichern der Ergebnisse
 * @return mixed Liefert die globale Variable
 */
function mxGetUserSession()
{
    global $cookie, $user_prefix; // global $cookie muss drin bleiben fuer Original nuke-Module !!
    if (!MX_IS_USER) {
        return array();
    }
    static $stat_cookie;
    if (!isset($stat_cookie)) {
        $user0 = base64_decode(mxSessionGetVar("user"));
        $stat_cookie = explode(":", $user0);
        if (empty($stat_cookie[0]) || empty($stat_cookie[1]) || empty($stat_cookie[2]) || !is_numeric($stat_cookie[0])) {
            unset($stat_cookie);
            $stat_cookie = array();
        }
    }
    $cookie = $stat_cookie; // benoetigt fuer Original nuke-Module !!
    return $cookie;
}

/**
 * setzt die benoetigten User-Informationen in die Session
 * bei Bedarf auch in den alten nuke-usercookie
 * Reihenfolge der Argumente:
 * uid, uname, pass, storynum, umode, uorder, thold, noscore, ublockon, theme, commentmax
 */
function mxSetUserSession()
{
    $args = func_get_args();
    $info = implode(":", $args);
    $info = base64_encode($info);
    mxSessionSetVar("user", $info);
    mxSetNukeCookie("user", $info, 1);
    mxSessionSafeCookie(MX_SAFECOOKIE_NAME_USER, 1);
    mxSessionSetVar('lasttime', 0); // online() ausf?hren
    mxSessionSetVar('user_uid', intval($args[0]));
    mxSessionSetVar('user_uname', $args[1]);
}

/**
 * Abfragen der User-Daten des aktuellen Users.
 *
 * @staticvar mixed $myuserinfo Variable zum Speichern der Ergebnisse
 * @param bool $forceDB Mit dem optionalen Parameter kann die Datenbankabfrage erzwungen werden
 * @return mixed Liefert ein Array mit den Userdaten des aktuellen Users zurueck
 */
function mxGetUserData($forceDB = 0)
{
    global $user_prefix;
    static $myuserinfo;
    if (isset($myuserinfo) && !$forceDB) {
        return $myuserinfo;
    }
    $userinfo = array();
    if (MX_IS_USER) {
        $user2 = base64_decode(mxSessionGetVar("user"));
        $user3 = explode(":", $user2);
        $GLOBALS['mxSkipSqlDetect'] = true;
        $result = sql_query("select *, (YEAR(CURRENT_DATE) - YEAR(user_bday)) - ( RIGHT(CURRENT_DATE,5) < RIGHT(user_bday,5)) AS user_age from ${user_prefix}_users where uname='" . mxAddSlashesForSQL(substr($user3[1], 0, 25)) . "' and pass='" . mxAddSlashesForSQL(substr($user3[2], 0, 32)) . "' and user_stat=1");
        if (is_resource($result)) {
            $userinfo = sql_fetch_assoc($result);
            sql_free_result($result);
            if (!empty($userinfo['uid'])) {
                $past = time() - MX_SETINACTIVE_MINS ;
                $userinfo['user_age'] = (empty($userinfo['user_bday']) || $userinfo['user_bday'] == '0000-00-00') ? 0 : (int)$userinfo['user_age'];
                $userinfo['user_online'] = true;
                $userinfo["user_ingroup"] = (empty($userinfo["user_ingroup"])) ? 1 : $userinfo["user_ingroup"];
            }
        }
    }
    $myuserinfo = $userinfo; // static !!
    return $userinfo;
}

/**
 * Abfragen der User-Daten von einem bestimmten User mit Usernamen als Abfrageparameter
 *
 * @static mixed $myuserinfo Variable zum Speichern der Ergebnisse
 * @param string $username Name des Users dessen Daten gebraucht werden
 * @param bool $user_stat Status des Users
 * @return mixed Liefert ein Array mit den Userdaten des Users zurueck
 */
function mxGetUserDataFromUsername($username, $user_stat = 1)
{
    global $user_prefix;
    static $myuserinfo, $lastname;
    if (isset($myuserinfo) && $lastname == $username) {
        return $myuserinfo;
    }
    $username = substr($username, 0, 25);
    $userinfo = array();
    $result = sql_query("select *, (YEAR(CURRENT_DATE) - YEAR(user_bday)) - ( RIGHT(CURRENT_DATE,5) < RIGHT(user_bday,5)) AS user_age from ${user_prefix}_users where uname='" . mxAddSlashesForSQL(substr($username, 0, 25)) . "' and user_stat=" . intval($user_stat));
    if (is_resource($result)) {
        $userinfo = sql_fetch_assoc($result);
        sql_free_result($result);
        if (!empty($userinfo['uid'])) {
            $past = time() - MX_SETINACTIVE_MINS ;
            $userinfo['user_age'] = (empty($userinfo['user_bday']) || $userinfo['user_bday'] == '0000-00-00') ? 0 : (int)$userinfo['user_age'];
            $userinfo['user_online'] = (($userinfo['user_lastvisit'] >= $past) && ($userinfo['user_lastmod'] != 'logout')) ? true : false;
            $userinfo["user_ingroup"] = (empty($userinfo["user_ingroup"])) ? 1 : $userinfo["user_ingroup"];
        }
    }
    $lastname = $username;
    $myuserinfo = $userinfo; // static !!
    return $userinfo;
}

/**
 * Abfragen der User-Daten von einem bestimmten User mit Userid als Abfrageparameter
 *
 * @static mixed $myuserinfo Variable zum Speichern der Ergebnisse
 * @param int $uid Userid des Users dessen Daten gebraucht werden
 * @param bool $user_stat Status des Users
 * ansonsten werden die Daten aus dem statischen Array zurueckgegeben
 * @return mixed Liefert ein Array mit den Userdaten des Users zurueck
 */
function mxGetUserDataFromUid($uid, $user_stat = 1)
{
    global $user_prefix;
    $uid = (int)$uid;
    static $myuserinfo, $lastuid;
    if (isset($myuserinfo) && $lastuid == $uid) {
        return $myuserinfo;
    }
    $userinfo = array();
    $GLOBALS['mxSkipSqlDetect'] = true;
    $result = sql_query("select *, (YEAR(CURRENT_DATE) - YEAR(user_bday)) - ( RIGHT(CURRENT_DATE,5) < RIGHT(user_bday,5)) AS user_age from ${user_prefix}_users where uid=" . $uid . " and user_stat=" . intval($user_stat));
    if (is_resource($result)) {
        $userinfo = sql_fetch_assoc($result);
        sql_free_result($result);
        if (!empty($userinfo['uid'])) {
            $past = time() - MX_SETINACTIVE_MINS ;
            $userinfo['user_age'] = (empty($userinfo['user_bday']) || $userinfo['user_bday'] == '0000-00-00') ? 0 : (int)$userinfo['user_age'];
            $userinfo['user_online'] = (($userinfo['user_lastvisit'] >= $past) && ($userinfo['user_lastmod'] != 'logout')) ? true : false;
            $userinfo["user_ingroup"] = (empty($userinfo["user_ingroup"])) ? 1 : $userinfo["user_ingroup"];
        }
    }
    $lastuid = $uid;
    $myuserinfo = $userinfo; // static !!
    return $userinfo;
}

/**
 * Dekodiert den "Admincookie" des aktuell angemeldeten Admins
 * Beispiel Aufruf: extract(mxGetAdminSession());
 *
 * @static mixed $data Variable zum Speichern der Ergebnisse
 * @return mixed Liefert ein Array mit den Ergebnissen
 */
function mxGetAdminSession()
{
    static $data;
    if (isset($data)) {
        return $data;
    }
    if (MX_IS_ADMIN) {
        $admin = base64_decode(mxSessionGetVar('admin'));
        $admin = explode(':', $admin);
        $data['aid'] = $admin[0];
        $data['pwd'] = $admin[1];
        $data['admlanguage'] = $admin[2];
    } else {
        $data['aid'] = '';
        $data['pwd'] = '';
        $data['admlanguage'] = $GLOBALS['currentlang'];
    }
    return $data;
}

/**
 * Liest alle Administratoreigenschaften des
 * aktuell angemeldeten Admins aus der Tabelle
 * Aufrufbeispiel: $arrayadmin = mxGetAdminData();
 *
 * @static mixed $data Variable zum Speichern der Ergebnisse
 * @return mixed Liefert ein Array mit den Ergebnissen
 */
function mxGetAdminData()
{
    static $data;
    if (!isset($data)) {
        global $prefix;
        $admin = mxGetAdminSession();
        if (empty($admin['pwd'])) {
            $data = array();
        } else {
            $GLOBALS['mxSkipSqlDetect'] = true;
            $result = sql_query("select * from ${prefix}_authors where aid='" . mxAddSlashesForSQL(substr($admin['aid'], 0, 25)) . "'");
            $data = sql_fetch_assoc($result);
            sql_free_result($result);
        }
    }
    // mxdebugfuncvars($admin);
    // mxdebugfuncvars($data);
    return $data;
}

/**
 * Prueft ob der aktuell angemeldete
 * Admin eine bestimmte Berechtigung hat
 * Aufrufbeispiel: $xx = mxGetAdminPref('radminarticle');
 *
 * @param string $preference Berechtigung, die abgefragt werden soll
 * @return bool Liefert das Ergebnis (true/false)
 */
function mxGetAdminPref($preference)
{
    $allprefs = mxGetAdminData();
    return (empty($allprefs[$preference]) && empty($allprefs['radminsuper'])) ? false : true;
}

/**
 * Ermittelt ob ein User online ist
 *
 * @param string $uname Username des Users fuer den der Onlinestatus abgefragt werden soll
 * @return bool Liefert das Ergebnis (true/false)
 */
function mxIsUserOnline($uname)
{
    global $prefix, $user_prefix;
    if (MX_IS_USER) {
        $usersession = mxGetUserSession();
        if (strtolower($usersession[1]) == strtolower($uname)) return true;
    }
    $past = time() - MX_SETINACTIVE_MINS ;
    $result = sql_query("SELECT count(uid) FROM ${user_prefix}_users AS u WHERE ((u.uname='" . mxAddSlashesForSQL(substr($uname, 0, 25)) . "') AND (u.user_lastvisit >= " . $past . ") AND (u.user_stat=1) AND (u.user_lastmod<>'logout'))");
    list($isonline) = sql_fetch_row($result);
    sql_free_result($result);
    return ($isonline) ? true : false;
}

/**
 * ermittelt ob der aktuelle User die Berechtigung hat, ein Modul zu nutzen
 *
 * @param string $modulname Modulname des Modules fuer das die Berechtigung geprueft werden soll
 * @staticvar mixed $allmodules Statische Variable in der beim ersten Aufruf der Funktion
 * mxModuleAllowed()
 * die Berechtigungen gespeichert werden
 * @return bool Liefert das Ergebnis (true/false)
 */
function mxModuleAllowed($modulename)
{
    global $prefix;
    static $allmodules; // statisches Array
    if (!isset($allmodules)) { // wenn statisches Array noch nicht initialisiert
        $allmodules = array(); // statisches Array initialisieren
        if (MX_IS_USER) { // ist User ?
            $userinfo = mxGetUserData(); // gesamte Userdaten in Array lesen
            $user_acl = (int)$userinfo["user_ingroup"]; // Gruppenzugehoerigkeit auslesen
        }
        $user_acl = (empty($user_acl)) ? 1 : $user_acl; // Gruppenzugehoerigkeit nur bei angemeldeten
        $main_module = mxGetMainModuleName();
        // die (nuke) Standard-Modulberechtigungen feststellen und in ein array stellen
        $query_view[] = "(title = '" . $main_module . "')"; // Home-Modul
        $query_view[] = "(view = 0 AND active=1)"; // Alle Besucher
        if (!MX_IS_USER) $query_view[] = "(view = 3 AND active=1)"; // Anonyme Besucher
        if (MX_IS_USER) $query_view[] = "(view = 1 AND g.group_id=" . $user_acl . " AND active=1)"; // Nur angemeldete Benutzer
        if (MX_IS_ADMIN) $query_view[] = "(view = 2)"; // Nur Administratoren
        // aus dem array mit Berechtigungen eine Abfragebedingung konstruieren
        $queryview = implode(" OR ", $query_view);
        // die aus den Bedingungen zusammengesetzte SQL-Abfrage
        $qry = "SELECT m.mid, m.title
             FROM ${prefix}_modules AS m LEFT JOIN ${prefix}_groups_modules AS g ON m.mid = g.module_id
                         WHERE ($queryview)
                         ORDER BY m.title ASC";
        $GLOBALS['mxSkipSqlDetect'] = true;
        $result = sql_query($qry); // sql ausfuehren
        if (is_resource($result)) { // wenn erfolgreich
            // Schleife, alle Moduldaten in array lesen
            while (list($mid, $title) = sql_fetch_row($result)) {
                // jetzt endlich das statische Array mit den Moduldaten fuellen
                $allmodules[$title] = 1;
            }
        }
        sql_free_result($result);
    }
    if (file_exists('modules/' . $modulename)) {
        return (empty($allmodules[$modulename])) ? 0 : 1;
    } else {
        $allmodules[$modulename] = 0;
        return 0;
    }
}

/**
 * Ermitteln des Modulnamens fuer die Startseite
 * wenn keines angegeben wurde, News als Startseite eintragen
 *
 * @static string $main Statische Variable in der der name des Stratseitenmodules gespeichert
 * wird
 * @return bool Liefert den Namen des Startseitenmodules zurueck
 */
function mxGetMainModuleName()
{
    global $prefix;
    static $main;
    if (!isset($main)) {
        $GLOBALS['mxSkipSqlDetect'] = true;
        $result = sql_query("select main_module from ${prefix}_main WHERE main_module <> ''");
        list($main) = sql_fetch_row($result);
        sql_free_result($result);
    }
    $main = (empty($main)) ? "News" : $main;
    if (!@file_exists("modules/" . $main . "/index.php")) {
        $main = "";
        $handle = @opendir("modules");
        if (!$handle) die("<h1>Error: the modules-folder is missing!</h1>");
        while ($dir = readdir($handle)) {
            if (!ereg("[.]", $dir)) {
                if (@file_exists("modules/" . $dir . "/index.php")) {
                    $main = $dir;
                    break;
                }
            }
        }
        closedir($handle);
        if (!empty($main)) {
            $GLOBALS['mxSkipSqlDetect'] = true;
            sql_query("DELETE FROM ${prefix}_main");
            $GLOBALS['mxSkipSqlDetect'] = true;
            sql_query("INSERT INTO ${prefix}_main (main_module) VALUES ('" . mxAddSlashesForSQL($main) . "')");
        }
    }
    return $main;
}

/**
 * Stellt die Blockdaten zusammen
 *
 * @param array $block
 * @param bool $nocache optional, default value false
 * @return array
 */
function mxGetBlockData($block, $nocache = false)
{
    global $prefix;

    if (!empty($block["url"])) {
        $past = time() - $block['refresh'];
        if ($block['time'] < $past) {
            $block['content'] = mxGetRssContent($block['url']);
            $GLOBALS['mxSkipSqlDetect'] = true;
            $result = sql_query("update ${prefix}_blocks set content='" . mxAddSlashesForSQL($block['content']) . "', time='" . time() . "' where bid='" . $block['bid'] . "'");
            if (MX_IS_ADMIN && (empty($block['content']) || !$result)) {
                $block['content'] = _RSSPROBLEM . '<br><a href="' . $block['url'] . '" target="_blank">' . mxCutString($block['url'], 40, '..', '/') . '</a>';
            }
        }
    } else if (!empty($block["blockfile"])) {
        $file = 'blocks/' . $block['blockfile'];
        if (!@file_exists($file)) {
            // / wenn blockfile fehlt, bei Admin Fehlermeldung zeigen
            if (MX_IS_ADMIN) {
                $block["content"] = _BLOCKPROBLEM . "<br>" . $block["blockfile"];
            }
        } else {
            if (empty($block["refresh"]) || $nocache) {
                $content = '';
                unset($blockfiletitle);
                // eventuelle Fehl-Ausgaben abfangen und zwischenspeichern
                ob_start();
                include($file);
                $block["content"] = trim(ob_get_clean()) . $content;

                if (isset($blockfiletitle)) {
                    $block['title'] = $blockfiletitle;
                }
            } else {
                $block = mxCacheBlocks($block);
            }
        }
    }
    return $block;
}

/**
 * Beschreibung
 *
 * @param string $action optional, default value ""
 * @return
 */
function mxYoubad($action = "")
{
    if (!empty($doblock)) mxSessionSetVar("blocked", 1);
    mxErrorScreen(_ERR_YOUBAD);
    $subject = "Bad work on my site";

    $action = (empty($action)) ? $subject : $action;

    $message = "Aktion: " . $action . "
    IP : " . MX_REMOTE_ADDR . "
    port: " . $_SERVER['REMOTE_PORT'] . "
    hostname : " . gethostbyaddr(MX_REMOTE_ADDR) . "
    time: " . date("Y-m-d h:i:s") . "
    browser: " . MX_USER_AGENT . "
    data: " . serialize($_REQUEST);

    mxMail($GLOBALS['adminmail'], $subject, $message, $GLOBALS['adminmail'], "text", "Mail-Bad");
    mxSecureLog("badrequest", $action, "with_request");
}

/**
 * Zum erstellen der Bildinformationen
 * mxCreateImage("themes/$thename/images/logo.gif", "alternate Text",
 * "border", "moreparameter");
 *
 * @param string $image
 * @param string $alt optional, default value ""
 * @param integer $border optional, default value 0
 * @param string $more optional, default value ""
 * @param integer $xhtml optional, default value 0
 * @return string
 */
function mxCreateImage($image, $alt = "", $border = 0, $more = "", $xhtml = 0)
{
    $border = (int)$border;
    @$size = GetImageSize($image);
    $para = "";
    if (empty($alt)) {
        $pathinfo = pathinfo($image);
        $alt = $pathinfo["basename"];
        $para .= "alt=\"" . $alt . "\"";
    } else {
        $para .= "alt=\"" . $alt . "\" title=\"" . $alt . "\"";
    }
    if (empty($size[3])) {
        if (MX_IS_ADMIN) {
            return "<span class=\"tiny\">missing image:<br />" . $image . "</span>";
        }
        return $alt;
    }
    if ($xhtml) {
        $xhtml = ' /';
    } else {
        $xhtml = '';
    }
    // $para .= "alt=\"$alt\"";
    $para .= " border=\"" . $border . "\"";
    return "<img src=\"$image\" $size[3] ${para} ${more}${xhtml}>";
}

/**
 * Kuerzt einen String ($string) auf eine bestimmte Laenge ($len)
 * und fuegt einen beliebigen String ($add) an das Ende
 *
 * @param string $string Variable dessen Inhalt gekuerzt werden soll
 * @param int $len Anzahl der Zeichen ab denen gekuerzt
 * @param string $add Zeichen welche an den gekuerzten String angehaengt werden sollen
 * @param string $cutter Trennzeichen#
 * @return string Liefert den gekuerzten String zurueck
 */
function mxCutString($string, $len, $add = "..", $cutter = " ")
{
    if (strlen($string) > $len) {
        $string = substr ($string, 0, $len + strlen($add));
        $last = (empty($cutter)) ? $len : strrpos ($string, $cutter);
        $last = (empty($last)) ? $len : $last;
        $string = trim(substr($string, 0, $last)) . $add;
    }
    return $string;
}

/**
 * kann verwendet werden um Variablen und Arrays in der HTML-Seite auszugeben
 * nur zum debuggen !!
 */
function mxDebugFuncvars()
{
    echo "<pre style='font-size: 11px; text-align: left; background: #FCFCFC; color: Black;'>\n";
    $arg_list = func_get_args();
    $numargs = func_num_args();
    for ($i = 0; $i < $numargs; $i++) {
        if (is_array($arg_list[$i]) || is_object($arg_list[$i])) {
            echo "Argument <b>$i</b> : ";
            ob_start();
            print_r($arg_list[$i]);
            $out = ob_get_contents();
            ob_end_clean();
            echo wordwrap(htmlentities($out), 200);
        } else {
            if (is_string($arg_list[$i])) $arg_list[$i] = wordwrap(htmlentities($arg_list[$i]), 200);
            echo "Argument <b>$i</b> : " . $arg_list[$i] . "\n";
        }
    }
    echo "</pre>";
}

/**
 * Debug-Funktion
 * by bernyregeling at hotmail dot com
 * found: http://de.php.net/manual/de/function.debug-backtrace.php
 *
 * @param bool $print optional, default value true
 * @return
 */
function mxDebugBacktrace($print = true)
{
    if (!function_exists('debug_backtrace')) {
        return '';
    }
    $olderr = error_reporting();
    error_reporting(0);
    $MAXSTRLEN = 64;
    $s = 'details: <pre style="font-family: \'Courier New\', Courier, monospace; font-size: 11px;">';
    $traceArr = debug_backtrace();
    array_shift($traceArr);
    foreach ($traceArr as $arr) {
        if (isset($arr['class'])) $s .= $arr['class'] . '.';
        foreach($arr['args'] as $v) {
            if (is_null($v)) $args[] = 'null';
            else if (is_array($v)) $args[] = 'Array[' . sizeof($v) . ']';
            else if (is_object($v)) $args[] = 'Object:' . get_class($v);
            else if (is_bool($v)) $args[] = $v ? 'true' : 'false';
            else {
                $v = (string) @$v;
                $str = htmlspecialchars(substr($v, 0, $MAXSTRLEN));
                if (strlen($v) > $MAXSTRLEN) $str .= '...';
                $args[] = $str;
            }
        }
        $s .= sprintf("file: %s # line:%4d", @$arr['file'], @$arr['line'], @$arr['file']);
        $s .= ', cmd: ' . $arr['function'] . '(' . implode(', ', $args) . '),';
        $s .= "\n";
    }
    $s .= '</pre>';
    $s = str_replace('\\', '/', str_replace(PMX_REAL_BASE_DIR, '', $s));
    if ($print) echo $s;
    error_reporting($olderr);
    return $s;
}

/**
 * Beschreibung
 *
 * @param string $url
 * @return string
 */
function mxCutHTTP($url)
{
    $url = trim($url);
    if ($url == "http://" || $url == "https://" || $url == "") return "";
    return (preg_match('#https?://#i', $url)) ? $url : "http://$url";
}

/**
 * Wertet RSS-Feeds aus und bereitet die Daten zur Anzeige vor
 *
 * @uses RSS_feed
 * @param string $url URL von dem RSS-Feed der angezeigt werden soll
 * @return string Liefert die aufbereiteten daten zurueck
 */
function mxGetRssContent($url)
{
    $url = mxCutHTTP($url);
    // zur Sicherheit, die entities richtigstellen
    $url = str_replace('&', '&amp;', str_replace('&amp;', '&', $url));
    include_once(PMX_SYSTEM_DIR . '/class_RSS_feed.php');
    $feed_parser = new RSS_feed();
    $feed_parser->Set_URL($url);
    // die Konfigwerte die an die Klasse ?bermittelt werden, sollte auch noch im Konfig-File eingebunden werden
    $feed_parser->Set_Limit("0"); // 0 = Default = Alle. Anzahl der Eintraege in der Liste
    $feed_parser->Show_Image(false); // Default = false. Zeige Bilddaten (bei hoeheren Versionen als 0.91)
    $feed_parser->Show_Description(false); // Default = false. Zeige Description-Data (bei hoeheren Versionen als 0.91)
    // Ende der Konfiguration
    $content = $feed_parser->Get_Results();
    // Ausgabe aufbereiten
    preg_match('#(.*)<ul>(.*)</ul>#is', $content, $parts);
    if (empty($parts[2])) {
        return '';
    }
    $content = preg_replace('#<li>(.*)<br\s*/*>#isU', '<div style="text-indent: -8px; margin-left: 8px;"><strong><big>&middot;</big></strong>&nbsp;$1</div>' . "\n", $parts[2]);
    if ($content) {
        $content .= '<div align="right"><br /><a href="' . $url . '" target="new" title="' . strip_tags($parts[1]) . '"><b>' . _HREADMORE . '</b></a></div>';
    }
    return $content;
}

/**
 * Beschreibung
 *
 * @param string $blockname
 * @return string
 */
function mxIsNewsBlockfile($blockname)
{
    return strstr($blockname, MX_NEWSBLOCK_PREFIX);
}

/**
 * Setzt oder loescht einen Cookie mit korrekten Parametern
 * entsprechend den Einstellungen des Session-Cookies
 *
 * @param string $cookiename Name fuer den zu erstellenden Cookie
 * @param string $cookievalue Wert fuer den Cookie
 * @param string $cookietime Gueltigkeitsdauer des Cookie's
 * Wenn $cookietime nicht gesetzt ist, wird der cookie beim beenden des scripts geloescht
 * Wenn $cookietime < 0 wird der Cookie sofort geloescht und wenn $cookietime == 1 wird die
 * $cookietime auf die Session-Dauer gesetzt . Ansonsten ird die uebergebene Zeit verwendet
 */
function mxSetCookie($cookiename, $cookievalue, $cookietime)
{
    $CookieInfo = session_get_cookie_params();
    $cookietime = (int)$cookietime;
    if (empty($cookietime)) { // beim beenden loeschen
        $cookietime = 0;
    } else if ($cookietime == 1) { // Dauer der session-lifetime
        $cookietime = time() + (int)MX_COOKIE_LIFETIME;
    } else if ($cookietime < 0) { // sofort loeschen
        $cookietime = time() - (int)MX_COOKIE_LIFETIME;
    }
    // ansonsten wird Dauer der angegebenen Zeit angenommen
    if ((empty($CookieInfo['domain'])) && (empty($CookieInfo['secure']))) {
        setcookie($cookiename, $cookievalue, $cookietime, $CookieInfo['path']);
    } else if (empty($CookieInfo['secure'])) {
        setcookie($cookiename, $cookievalue, $cookietime, $CookieInfo['path'], $CookieInfo['domain']);
    } else {
        setcookie($cookiename, $cookievalue, $cookietime, $CookieInfo['path'], $CookieInfo['domain'], $CookieInfo['secure']);
    }
}

/**
 * Setzt oder loescht die alten Cookies von phpNuke
 * wenn in config.php entsprechend eingestellt
 * Dies kann fuer bestimmte phpNuke-Module noetig sein
 *
 * @param string $CookieName Name fuer den zu erstellenden Cookie
 * @uses mxSetCookie()
 * @param string $CookieValue Wert fuer den Cookie
 * @param string $inout Gueltigkeitsdauer des Cookie's
 * Wenn $inout nicht gesetzt ist, wird der Cookie sofort geloescht
 * Wenn $inout == 1 wird die Lebensdauer des Cookie auf die Session-Dauer gesetzt.
 */
function mxSetNukeCookie($CookieName, $CookieValue = "", $inout = 0)
{
    if (empty($GLOBALS['mxCreateNukeCookie'])) return;
    $inout = (empty($CookieValue)) ? 0 : $inout;
    $CookieTime = ($inout == 1) ? 1 : -1;
    mxSetCookie($CookieName, $CookieValue, $CookieTime);
}

/**
 * Setzen von File-Permissions (chmods)
 *
 * @param string $file Angabe des Files dessen Berechtigungen geaendert werden sollen (kopmletter
 * Pfad)
 * @param string $mode Modus lock ergibt chmod 444, unllock ergibt chmod 777)
 * @return bool Liefert true wenn die Aktion erfolgreich war, ansonsten false
 */
function mxChangeFilePerms($file, $mode)
{
    if (!file_exists($file)) {
        return false;
    }
    $err = false;
    if ($mode == 'lock') {
        @chmod($file, 0444);
        $err = (is_writable($file));
    } else if ($mode == 'unlock') {
        @chmod($file, 0777);
        $err = (!is_writable($file));
    }
    return (isset($php_errormsg) || $err) ? false : true;
}

/**
 * Prueft ob der User Fotos bzw. Avatare hochgeladen hat.
 * Aufrufbeispiele:
 * $foo = mxIsAvatarUploaded($bar);->false oder den aktuellen Avatar des Users als String
 * $foo = mxIsAvatarUploaded($bar, true);->false oder den aktuellen Avatar des Users als
 * mxCreateImage.
 * $foo = mxIsAvatarUploaded($bar,false,true);->false oder $row als Array mit Zugriff auf alle
 * Spalten (z.B $foo['werist'])
 *
 * @param string $avatar_user_name Username von dem User desses Avatar gesucht wird
 * @param bool $imagereturn Wenn imareturn==true wird der komplette HTML-Tag zum Anzeigen des
 * Avatars zurueckgegeben, ansonsten nur der Avatar Dateiname, optional, default value false
 * @param bool $new Wenn $new==true wird ein Array mit den gesammten daten zum Useravatar
 * zurueckgegeben, , optional, default value false
 * @return mixed Liefert Image, Avatar-Dateinamen oder Array mit Informationen zum Avatar je nach
 * uebergebenen Parametern
 */
function mxIsAvatarUploaded($avatar_user_name, $imagereturn = false, $new = false)
{
    // Wenn das Modul installiert ist:
    if (@is_file('modules/Avatar/index.php')) {
        global $prefix;
        $ergebnis = sql_query("SELECT * FROM ${prefix}_avatar WHERE `aid`='" . mxAddSlashesForSQL($avatar_user_name) . "'");
        $row = sql_fetch_assoc($ergebnis);
        if ($new) {
            return(!empty($row))? $row:false;
        } else {
            if ($imagereturn)
                return (empty($row['werist'])) ? false : mxCreateImage(MX_PATH_MEMBERAVATAR . "/" . $row['werist'], $avatar_user_name);
            else
                return (empty($row['werist'])) ? false : $row['werist'];
        }
    } else
        return false;
}

/**
 * Funktion zum loeschen aller Avatare und Einstellungen eines Users
 *
 * @param string $avatar_user_name Username des User desses Avatar geloescht werden soll
 * @return
 */
function mxDeleteAvatar($avatar_user_name)
{
    if (mxModuleAllowed('Avatar')) {
        global $prefix;
        $ergebnis = sql_query("SELECT aid, typ, id FROM ${prefix}_avatar WHERE `aid`='" . mxAddSlashesForSQL($avatar_user_name) . "'");
        list($aid, $typ, $id) = sql_fetch_row($ergebnis);
        $qry = "DELETE FROM ${prefix}_avatar WHERE `aid`='" . mxAddSlashesForSQL($avatar_user_name) . "'";
        $qry2 = "DELETE FROM ${prefix}_avatar_comments WHERE `alid`='" . mxAddSlashesForSQL($id) . "'";
        sql_query($qry);
        sql_query($qry2);
        $clean_file = explode("|", $typ);
        foreach($clean_file as $new_file) {
            if (is_file(MX_PATH_MEMBERAVATAR . "/" . $new_file)) {
                @unlink(MX_PATH_MEMBERAVATAR . "/" . $new_file);
            }
        }
    } else {
        return false;
    }
}

/**
 * Rekonvertierung von html_entities in Umlaute
 * Funktion ist ab PHP 4.3.0 in PHP enthalten ABER macht Probleme!
 *
 * @param string $string Text, welcher HTML-Entitaeten enthalten kann
 * @return string konvertierter Text, in dem die Enttitaeten wieder in normale Umlaute zurueckgewandelt
 * sind
 */
function mxHtmlEntityDecode($string)
{
    $string = strtr($string, array_flip(get_html_translation_table(HTML_ENTITIES)));
    $string = preg_replace("/&#([0-9]+);/me", "chr('\\1')", $string);
    return $string;
}

/**
 * Aktivieren und Deaktivieren des Bulk-Mail Modus fuer die Funktion mx_mail()
 *
 * @param bool $what Zum Einschalten des Bulkmailings ==1, zum Ausschalten ==0
 * @param string $mode - Bulmail Versandmodus (nur mit Sendmail & EXIM kompatibelen MTA
 * bei allen anderen MTA wird ein anderer Wert als 'q' einen Fehler provozieren)
 * Moegliche Werte fuer $modus sind :
 * SENDMAIL_DELIVERY_DEFAULT = ''
 * Does not override the default mode.
 *
 * SENDMAIL_DELIVERY_INTERACTIVE - 'i'
 * Attempt to send the messages synchronously to the recipient's SMTP server and only returns when
 * it succeeds or fails. This is usually the default mode. It stalls the delivery of messages but it may be
 * safer to preserve disk space because the successfully delivered messages are not stored.
 * SENDMAIL_DELIVERY_BACKGROUND - 'b'
 * Creates a background process that attempts to deliver the message and returns immediately. This
 * mode is recommended when you want to send a few messages as soon as possible. It is not recommended for
 * sending messages to many recipients as it may consume too much memory and CPU that result from creating
 * excessive background processes.
 * SENDMAIL_DELIVERY_QUEUE - 'q'
 * Just drop the message in the queue and leave it there until next time the queue is run. It is
 * recommended for deliverying messages to many recipients as long as there is enough disk space to store
 * all the messages in the queue.
 * SENDMAIL_DELIVERY_DEFERRED - 'd'
 * The same as the queue mode except for a few verifications that are skipped.
 */
function mxBulkMail($what = 0, $mode = 'q')
{
    if ((!@is_file(PMX_SYSTEM_DIR . "/mailclasses/email_message.php")) || (!@is_file(PMX_SYSTEM_DIR . "/mailclasses/sendmail_message.php"))) {
        return false;
    } else {
        require_once(PMX_SYSTEM_DIR . '/mailclasses/email_message.php');
        require_once(PMX_SYSTEM_DIR . '/mailclasses/smtp_message.php');
        require_once(PMX_SYSTEM_DIR . '/mailclasses/smtp.php');
        require_once(PMX_SYSTEM_DIR . '/mailclasses/sasl.php');
        $email_message = new smtp_message_class;
        $email_message->SetBulkMail($what);
        $email_message->bulk_mail_delivery_mode = $mode;
    }
}

/**
 * Versenden von Text und HTML-Mails mit und ohne Datei-Anhaenge
 *
 * @uses smtp_message_class
 * @param string $to - Empfaengeradresse der Email
 * @param string $subject - Subject der Email
 * @param string $message - Mitteilung /Message (text oder Html)
 * @param string $mxsender - Absenderadresse der Mail (setzt Admin-mailadresse wenn nicht angegeben)
 * @param string $type - Type der Email (text or html)
 * @param string $seclogid - Bezeichner fuer das Security-Log
 * @param string $sendername - Absendername der Mail
 * @param int $wrap - Textumbruch nach diser Anzahl von Zeichen
 * @param mixed $bcc_array - Array fuer BCC-Empfaenger das Array folgendermassen fuellen :
 * $bcc_array=array('peter@gabriel.org' => 'Peter Gabriel'...)
 * @param mixed $the_attachment - Array : Dieses Array fuer File-Anhaenge uebergeben. Array muss
 * enthalten:
 * Inhalt fuer $the_attachment: "Name" = Dateiname in der Mitteilung,
 * "FileName" = URL zumr Datei, "Disposition" = "attachment", "Content-
 * Type" = "automatic/name" (Dies sollte den Filetype automatisch feststellen)
 */
function mxmail($to, $subject, $message, $mxsender = '', $type = 'text', $seclogid = '', $sendername = '', $wrap = 80, $bcc_array = '', $the_attachment = '')
{
    if (empty($to)) return false;
    if (empty($message) && empty($subject)) return false;
    if (empty($type)) $type = "text";

    $message = preg_replace("/(?<!\r)\n/si", "\r\n", $message);
    $to = ereg_replace('[[:cntrl:]|" "]', " ", $to);

    if ((!@is_file(PMX_SYSTEM_DIR . "/mailclasses/email_message.php")) || (!@is_file(PMX_SYSTEM_DIR . "/mailclasses/smtp_message.php")) || ($GLOBALS['mailauth'] == 'mail')) {
        $mxsender = (empty($mxsender)) ? trim($GLOBALS['adminmail']) : trim($mxsender);
        $charset = (defined('_CHARSET')) ? _CHARSET : 'iso-8859-1';
        if (strtoupper(substr(PHP_OS, 0, 3) == 'WIN')) {
            $eol = "\r\n";
        } elseif (strtoupper(substr(PHP_OS, 0, 3) == 'MAC')) {
            $eol = "\r";
        } else {
            $eol = "\n";
        }
        if ($type == "text") {
            $subject = strip_tags(mxHtmlEntityDecode($subject));
            $message = strip_tags(mxHtmlEntityDecode($message));
            $header = "From: " . $sendername . " <" . $mxsender . ">" . $eol;
            $header .= "Reply-To: " . $sendername . " <" . $mxsender . ">" . $eol;
            $header .= "X-Mailer: PHP v" . phpversion() . $eol;
            $header .= 'Content-type: text/plain; charset=' . $charset . $eol;
        } else {
            $header = "From: " . $sendername . " (" . $mxsender . ")" . $eol;
            $header .= "Reply-To: " . $sendername . " <" . $mxsender . ">" . $eol;
            $header .= "X-Mailer: PHP v" . phpversion() . $eol;
            $header .= "MIME-Version: 1.0" . $eol;
            $header .= 'Content-type: text/html; charset=' . $charset . $eol;
        }
        return @mail($to, $subject, $message, $header);
    } else {
        global $email_message;
        $from_name = (empty($sendername)) ? trim(ereg_replace("[:!\"$%&=]", "", $GLOBALS['sitename'])) : trim(ereg_replace("[:!\"$%&=]", "", $sendername));
        $from_address = (empty($mxsender)) ? trim($GLOBALS['adminmail']) : trim($mxsender);
        $reply_name = (empty($sendername)) ? trim(ereg_replace("[:!\"$%&=]", "", $GLOBALS['sitename'])) : trim(ereg_replace("[:!\"$%&=]", "", $sendername));
        $reply_address = (empty($mxsender)) ? trim($GLOBALS['adminmail']) : trim($mxsender);
        $error_delivery_name = (empty($sendername)) ? trim(ereg_replace("[:!\"$%&=]", "", $GLOBALS['sitename'])) : trim(ereg_replace("[:!\"$%&=]", "", $sendername));
        $error_delivery_address = (empty($mxsender)) ? trim($GLOBALS['adminmail']) : trim($mxsender);
        $to_name = "";
        $to_address = $to;
        $subject = $subject;
        $message = $message;
        if (!is_object($email_message)) {
            require_once(PMX_SYSTEM_DIR . '/mailclasses/email_message.php');
            require_once(PMX_SYSTEM_DIR . '/mailclasses/smtp_message.php');
            require_once(PMX_SYSTEM_DIR . '/mailclasses/smtp.php');
            require_once(PMX_SYSTEM_DIR . '/mailclasses/sasl.php');
            $email_message = new smtp_message_class;
        }
        $email_message->ResetMessage(); //reset komplete
        /**
         * This computer address
         */
        $email_message->localhost = "localhost";
        /**
         * SMTP server address, probably your ISP address
         */
        $email_message->smtp_host = $GLOBALS['mailhost'];
        /**
         * Deliver directly to the recipients destination SMTP server
         */
        $email_message->smtp_direct_delivery = 0;
        /**
         * In directly deliver mode, the DNS may return the IP of a sub-domain of
         * the default domain for domains that do not exist. If that is your
         * case, set this variable with that sub-domain address.
         */
        $email_message->smtp_exclude_address = "";
        /**
         * If you use the direct delivery mode and the GetMXRR is not functional,
         * you need to use a replacement function.
         */
        /**
         * $_NAMESERVERS=array();
         * include("rrcompat.php");
         * $email_message->smtp_getmxrr="_getmxrr";
         */
        /**
         * authentication user name
         */
        $email_message->smtp_user = $GLOBALS['mailuname'];
        /**
         * authentication realm or Windows domain when using NTLM authentication
         */
        $email_message->smtp_realm = "";
        /**
         * authentication workstation name when using NTLM authentication
         */
        $email_message->smtp_workstation = "";
        /**
         * authentication password
         */
        $email_message->smtp_password = $GLOBALS['mailpass'];
        /**
         * if you need POP3 authentication before SMTP delivery,
         * specify the host name here. The smtp_user and smtp_password above
         * should set to the POP3 user and password
         */
        $email_message->smtp_pop3_auth_host = $GLOBALS['popauth'];
        /**
         * Specify whether it should use secure connections with SSL to connect to the SMTP server.
         * Certain e-mail services like Gmail require SSL connections.
         */
        $email_message->smtp_ssl = 0; //Hier muss noch eine Einstellmglichkeit in das Admin-Men bei den Maileinstellungen $GLOBALS['USE_MAIL_SSL']
        /**
         * Specify the default character set to be assumed for the message headers and body text.
         * Change this variable to the correct character set name if it is different than the default.
         */
        $email_message->default_charset = (defined('_CHARSET')) ? _CHARSET : 'iso-8859-1';
        if ((!empty($GLOBALS['mxDebugService'])) && (MX_IS_ADMIN)) {
            /**
             * Output dialog with SMTP server
             */
            $email_message->smtp_debug = 1; //Debug-Mode on/off
            $email_message->smtp_html_debug = 1; //Show Debug-Info as HTML-Page
        } else {
            $email_message->smtp_debug = 0; //Debug-Mode on/off
            $email_message->smtp_html_debug = 0; //Show Debug-Info as HTML-Page
        }
        if ((isset($bcc_array)) && ($bcc_array != "") && (is_array($bcc_array))) {
            $email_message->SetMultipleEncodedEmailHeader('Bcc', $bcc_array);
        }
        $email_message->SetEncodedEmailHeader("To", $to_address, $to_name);
        $email_message->SetEncodedEmailHeader("From", $from_address, $from_name);
        $email_message->SetEncodedEmailHeader("Reply-To", $reply_address, $reply_name);
        $email_message->SetHeader("Return-Path", $error_delivery_address);
        $email_message->SetEncodedEmailHeader("Errors-To", $error_delivery_address, $error_delivery_name);
        $email_message->SetEncodedHeader("Subject", $subject);
        if ($type == "text") {
            $email_message->AddQuotedPrintableTextPart($email_message->WrapText($message, $wrap, ""));
        } else {
            $email_message->CreateQuotedPrintableHTMLPart($message, "", $html_part);
            $text_message = "This is an HTML message. Please use an HTML capable mail program to read this message.";
            $email_message->CreateQuotedPrintableTextPart($email_message->WrapText($text_message, $wrap, ""), "", $text_part);
            $alternative_parts = array($text_part,
                $html_part
                );
            $email_message->AddAlternativeMultipart($alternative_parts);
        }
        if (($the_attachment != "") && (is_array($the_attachment))) {
            $email_message->AddFilePart($the_attachment);
        }
        $error = $email_message->Send();
        for($recipient = 0, Reset($email_message->invalid_recipients);$recipient < count($email_message->invalid_recipients);Next($email_message->invalid_recipients), $recipient++)
        mxSecureLog("Mailsystem-Invalid recipient: ", Key($email_message->invalid_recipients) . " Error: " . $email_message->invalid_recipients[Key($email_message->invalid_recipients)] . "\n");
        if (!empty($seclogid)) {
            $securelog = $seclogid . "\n";
            mxSecureLog("Mailsystem-Log: ", $securelog . " " . $subject . " \n" . $to);
        }
        if (strcmp($error, "")) {
            mxSecureLog("Mailsystem-Error: ", $subject . " \n" . $to . " \n" . $error);
            return false;
        }
        unset ($alternative_parts);
    }
    return true;
}

/**
 * Funktion zum Auslesen des Geschlechts der angegebenen Person
 *
 * @param string $usernamen Name des Users dessen Geschlecht festgestellt werden soll
 * @return string Liefert das Geschlecht des Users zurueck
 */
function mxGetUserGeschlecht($username)
{
    static $user_sexus, $lastusername;
    if (isset($user_sexus) && $username == $lastusername) {
        return $lastsexus;
    }
    global $user_prefix;
    $ergebnis = sql_query("SELECT user_sexus FROM ${user_prefix}_users WHERE `uname`='" . mxAddSlashesForSQL(substr($username, 0, 25)) . "'");
    list($user_sexus) = sql_fetch_row($ergebnis);
    return $user_sexus;
}

/**
 * Funktion zum Auslesen des Gruppennamens anhand der Gruppen-ID
 *
 * @param  $groupid
 * @return string
 */
function mxGetGroupTitle($groupid)
{
    global $user_prefix, $prefix;
    $groupid = (empty($groupid)) ? 1 : $groupid;
    $GLOBALS['mxSkipSqlDetect'] = true;
    $result = sql_query("SELECT access_title FROM ${prefix}_groups_access WHERE access_id ='" . intval($groupid) . "'");
    list($grouptitle) = sql_fetch_row($result);
    return $grouptitle;
}

/**
 * Beschreibung
 *
 * @param string $dateformat
 * @param integer $unixtimestamp optional, default value 0
 * @return string
 */
function mx_strftime($dateformat, $unixtimestamp = 0)
{
    if (empty($unixtimestamp)) $unixtimestamp = time();
    $out = strftime($dateformat, $unixtimestamp);
    $search = array("January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December", "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday");
    $replace = array(_JANUARY, _FEBRUARY, _MARCH, _APRIL, _MAY, _JUNE, _JULY, _AUGUST, _SEPTEMBER, _OCTOBER, _NOVEMBER, _DECEMBER, _WEEKFIRSTDAY, _WEEKSECONDDAY, _WEEKTHIRDDAY, _WEEKFOURTHDAY, _WEEKFIFTHDAY, _WEEKSIXTHDAY, _WEEKSEVENTHDAY);
    $out = str_replace($search, $replace, $out);
    return $out;
}

/**
 * Gibt eine formatierte Datumsangabe zu einer uebergebene Zeitangabe zurueck
 *
 * @staticvar string $ger
 * @param int $time Zeitangabe
 * @return string Liefert je nach Spracheinstellung eine deutsche oder internationale Zeitangabe
 * zurueck
 */
function formatTimestamp($time, $dateformat = _DATESTRING)
{
    global $datetime; /// global $datetime muss drin bleiben fuer alte Module
    static $ger;
    if (!isset($ger)) {
        $ger = ereg("german", $GLOBALS["currentlang"]);
    }
    ereg("([0-9]{4})-([0-9]{1,2})-([0-9]{1,2})( ([0-9]{1,2}):([0-9]{1,2}):([0-9]{1,2}))?", $time, $datetime);
    if ($datetime[1] < 1970) {
        if ($ger) {
            $datetime = sprintf("%02d.%02d.%04d", $datetime[3], $datetime[2], $datetime[1]);
        } else {
            $datetime = sprintf("%04d-%02d-%02d", $datetime[1], $datetime[2], $datetime[3]);
        }
    } else {
        $datetime = mx_strftime($dateformat, mktime(intval($datetime[5]), intval($datetime[6]), intval($datetime[7]), $datetime[2], $datetime[3], $datetime[1]));
        $datetime = ucfirst($datetime);
    }
    return($datetime);
}

/**
 * Nur zur Kompatibilitaet mit phpNuke-Modulen
 *
 * @param int $timestamp Unix-Zeitstempel
 * @return Liefert phpNuke-konformes Registrierungsdatum zurueck
 */
function mxGetNukeUserregdate($timestamp = 0)
{
    $timestamp = (empty($timestamp)) ? time() : $timestamp;
    setlocale(LC_TIME, "en_EN");
    $user_regdate = date("M d, Y", $timestamp);
    setlocale(LC_TIME, $GLOBALS['locale']);
    return $user_regdate;
}

/**
 * SQL-Konformes Datum schreiben
 *
 * @param int $year Jahr
 * @param int $month Monat
 * @param int $day Tag
 * @return string Liefert SQL-Konformes Datum zurueck
 */
function mxGetSqlDate($year, $month, $day)
{
    $isdate = checkdate((int)$month, (int)$day, (int)$year);
    $out = ($isdate) ? sprintf("%04d-%02d-%02d", $year, $month, $day) : "";
    return $out;
}

/**
 * Beschreibung
 */
function mxDemoMode()
{
    global $mxDemoMode;
    // Demomode bei berechtigten Admins wieder abschalten
    static $check;
    if (isset($check, $GLOBALS['dbi'])) {
        return $check;
    }
    $check = $mxDemoMode;
    if ($check && isset($GLOBALS['dbi'])) {
        if (!defined('MX_DEMOMODE_RUN')) {
            define('MX_DEMOMODE_RUN', 1);
        }
        $aaa = mxGetAdminSession();
        if (in_array($aaa['aid'], $GLOBALS["mxDemoAdmins"])) {
            $check = 0;
        }
    }
    unset($GLOBALS["mxDemoAdmins"]);
    // $GLOBALS["mxDemoMode"] = $check;
    $mxDemoMode = $check;
    return $check;
}

/**
 * Weiterleitung zu einer anderen URL
 *
 * @param string $url URL zu der weitergeleitet werden soll
 * @param string $message Mitteilung die zur Weiterleitung angezeigt werden soll
 * @param int $delay Wartezeit bis zur Weiterleitung
 */
function mxRedirect($url, $message = "", $delay = 3)
{
    // if (ereg("(?!http://|https://|ftp://|mailto:)",$url)){
    if (empty($url)) {
        $url = "index.php";
    } else {
        if (!preg_match('!^http!', $url)) {
            // Removing leading slashes from redirect url
            $url = PMX_HOME_URL . '/' . ltrim($url, '/\\');
        }
    }
    @session_write_close();
    if ($message) {
        header("Refresh: $delay;url=" . str_replace('&amp;', '&', $url));
        mxRedirectMessage($url, $message, $delay);
    } else {
        header("Location: " . str_replace('&amp;', '&', $url));
    }
    die();
}

/**
 * Wandelt SQL-Konforme Zeitangabe in einen Unix-Zeitstempel um
 *
 * @param string $sqldate Zeitangabe aus Datenbankquerie
 * @return integer Liefert Unix-Zeitstempel zurueck
 */
function mxSqlDate2UnixTime($sqldate)
{
    if (empty($sqldate)) return 0;
    if (ereg("([0-9]{4})-([0-9]{1,2})-([0-9]{1,2}) ([0-9]{1,2}):([0-9]{1,2}):([0-9]{1,2})", $sqldate, $x)) {
        return mktime($x[4], $x[5], $x[6], $x[2], $x[3], $x[1]);
    }
    return 0;
}

/**
 * Wandelt Anfuehrungszeichen in deren Entitaeten um
 *
 * @param string $string Variable deren Inhalt umgewandelt werden soll
 * @param string $quote_style Umwandlungsmodus
 * 2 = ENT_COMPAT = behandelt nur die doppelten, nicht aber die einfachen Anfuehrungszeichen
 * 3 = ENT_QUOTES = werden einfache und doppelte Anfuehrungszeichen umgewandelt
 * 0 = ENT_NOQUOTES = sowohl einfache als auch doppelte Anfuehrungszeichen bleiben unberuehrt
 * @return string Liefert umgewandelten String zurueck
 */
function mxEntityQuotes($string, $quote_style = ENT_QUOTES)
{
    if ($quote_style == ENT_COMPAT) {
        return str_replace('"', '&quot;', $string);
    } else if ($quote_style == ENT_QUOTES) {
        return str_replace("'", '&#039;', str_replace('"', '&quot;', $string));
    }
    return $string;
    // mxDebugFuncVars(htmlentities('"', ENT_QUOTES), ENT_COMPAT, ENT_QUOTES, ENT_NOQUOTES);
}

/**
 * Email Address Verification
 * found at: http://www.devshed.com/c/a/PHP/Email-Address-Verification-with-PHP/1/
 *
 * @param string $email Emailadresse, die ueberprueft werden soll
 * @return boolean Liefert true bzw. im Fehlerfall false zurueck
 */
function mxCheckEmail($email)
{
    if (preg_match("/^([a-zA-Z0-9-])+([a-zA-Z0-9\._-])*@([a-zA-Z0-9_-])+([a-zA-Z0-9\._-]+)+$/", $email)) {
        return true;
    }
    return false;
}

/**
 * http://phpdiscovery.com/dangers-of-remote-execution/
 */
function cleaneval($specialchars)
{
    $bad = array('{', '}', '$', ']', '[', '`', ';');
    $good = array('&#123', '&#125', '&#36', '&#93', '&#91', '&#96', '&#59');
    $specialchars = htmlspecialchars($specialchars);
    $specialchars = str_replace($bad, $good, $specialchars);
    return $specialchars;
}

function cleanpreg($specialchars)
{
    $bad = array('{', '}', '$');
    $good = array('&#123', '&#125', '&#36;');
    $specialchars = htmlspecialchars($specialchars);
    $specialchars = str_replace($bad, $good, $specialchars);
    return $specialchars;
}

/**
 * Wandelt eine Integer- oder Fliesskommazahl in einen formatierten String um,
 * damit die Ausgabe in einer lesbareren Form geschieht.
 *
 * Die Formatierung stuetzt sich auf die ISO 31 (ab 2008: ISO 80000).
 * Info: http://de.wikipedia.org/wiki/Schreibweise_von_Zahlen#Internationale_Standards
 *
 * Wenn die Anzahl der auszugebenden Nachkommastellen kleiner ist als die
 * Anzahlzahl der Nachkommastellen des umzuwandelnden Wertes, wird beim Umwandeln
 * kaufmaennisch gerundet.
 *
 * Die Funktion dient ausschliesslich dazu, Zahlen in einer formatierten Form
 * zur Anzeige zu bringen. Alle mathematischen Berechnungen muessen vor
 * Verwendung der Funktion ausgefuehrt werden.
 *
 * @param mixed $value Zahl, die als formatierter String ausgegeben werden soll
 * @param integer $decimal_precision Anzahl der auszugebenden Nachkommastellen,
 * optional, default value 2
 * @return string $new_value Rueckgabe des Wertes als formatierter String.
 * Bei einem Fehler wird false zurueckgegeben.
 */
function mxValueToString($value, $decimal_precision = 2)
{
    $decimals_separator = ','; // Dezimaltrennzeichen
    /* Da es, insbesonder unter dem Betriebssystem Windows(tm), bei Verwendung
     * des schmalen geschuetzen Leerzeichen (&#x202F;) zu Darstellungsproblemen
     * kommen kann, wird das "normale" geschuetze Leerzeichen (&nbsp;) als
     * Trennzeichen der dargestellten Dreiergruppen verwendet.
     */
    $thousands_separator = '&nbsp;'; // Tausender- bzw. 3er-Gruppentrennzeichen
    if (is_numeric($value)) { // Wert muss eine Zahl sein
        $new_value = str_replace('.', $thousands_separator , number_format($value, $decimal_precision, $decimals_separator, '.'));
        if (stristr($new_value, $decimals_separator)) {
            /* Da die function number_format() nur den Teil vor dem Dezimaltrenner
             * mit einem Tausendertrennzeichen versehen kann, machen wir das
             * mir den Nachkommastellen jetzt manuell.
             */
            $temp_1 = substr(stristr($new_value, $decimals_separator), 1); // Nachkommateil extrahieren
            $length = strlen($temp_1); // Anzahl der Stellen
            if ($length > 3) {
                $quota = floor($length / 3); // Anzahl der moeglichen 3er Gruppen
                $temp_2 = '';
                $x = 0; // Startposition zjm Extrahieren der 3er Gruppen
                /* Immer 3 Zeichen plus einem Leerzeichen zu einem neuen String zusammenfuegen
                 * und zwar von links beginnend.
                 */
                for ($y = 0; $y < $quota; $y++) {
                    $temp_2 .= substr($temp_1, $x, 3) . ' ';
                    $x = $x + 3;
                }
                // Wenn noch Zeichen uebrig sind
                if (strlen(substr($temp_1, $x)) != 0) {
                    // dann hinten anfuegen
                    $temp_2 .= substr($temp_1, $x);
                } else {
                    // ansonsten das Leerzeichen am Ende wieder entfernen
                    $temp_2 = str_replace(' ', $thousands_separator, rtrim($temp_2));
                }
                // Den Teil vor dem Dezimaltrenner inkl. dem Dezimaltrenner extrahieren
                $temp_3 = substr($new_value, 0, (strpos($new_value, $decimals_separator) + 1));
                // Den vorhergehenden Extrakt mit den manuel formatiertem Rest zusammenfuegen
                $new_value = $temp_3 . $temp_2;
            }
        }
    } else {
        $new_value = false;
    }
    return $new_value;
}

?>
