<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 1.1.2.11 $
 * $Author: tora60 $
 * $Date: 2008/02/01 17:52:55 $
 */

/* Tidyfunktion, die sich direkt in die Spaw-HTML-Ausgabe einklinkt getHtml() */
function mxSpawTidy($output)
{
    // proprietaere Attribute entfernen
    $output = str_replace('unselectable="on"', '', $output);
    // angegebene Tabellenhoehe in style umschreiben
    $replaces = array();
    preg_match_all('#(<t(?:able|d|h)[^>]+)(?:height=[\'"]*([^\'"]+)[\'"]*)([^>]*>)#', $output, $matches);
    foreach ($matches[0] as $key => $value) {
        $replaces[$key] = $matches[1][$key] . ' style="height: ' . $matches[2][$key] . ';"' . $matches[3][$key];
    }
    if ($replaces) {
        $output = str_replace($matches[0], $replaces, $output);
    }
    // leeren Span-Tag von Toolbar entfernen
    $output = str_replace('<span style="white-space: nowrap;"></span>', '', $output);
    // den Stylesheet des Spaw-Themes an die richtige Stelle setzen
    if (class_exists('pmxHeader')) {
        if (preg_match_all('#(?:<link[^>]+>)|(?:<style[^>]+>.+</style>)#si', $output, $matches)) {
            $allstyles = array_unique($matches[0]);
            pmxHeader::add($allstyles);
            $output = str_replace($allstyles, '', $output);
        }
    }

    return $output;
}

/* Tidyfunktion, die sich direkt in die Spaw-HTML-Ausgabe der Dialogfenster einklinkt */
function mxSpawDialogTidy($output)
{
    // versch. Module und Addons schreiben einen <link> Tag in den Quelltext
    // dies ist kein html-Standard und wird hier versucht zu fixen....
    $tmp = explode('</head>', $output);
    if (isset($tmp[1]) && !isset($tmp[2])) {
        // wenn link-Tags nach dem head-Endetag vorkommen
        if (preg_match_all('#(?:<link[^>]+>)|(?:<style[^>]+>.+</style>)#si', $tmp[1], $matches)) {
            $allstyles = array_unique($matches[0]);
            // den html-Output behandelt neu zusammensetzen
            $output = str_replace($allstyles, '', $output);
            $output = str_replace('</head>', implode("\n", $allstyles) . "\n</head>", $output);
        }
    }
    // angehaengte microtime url-encoden
    $output = preg_replace('#(empty\/empty\.html\?)([0-9 \.]+)#e', '"\1" . urlencode("\2")', $output);
    // versch. falsche Attribute fixen
    $output = str_replace(' algin="', ' align="', $output);
    $output = str_replace(' itle="', ' title="', $output);
    $output = mxSpawTidy($output);
    return $output;
}

/* Klassen im Stylesheet ermitteln und als Array zurueckgeben*/
function mxSpawGetStyleContent($stylefile_abs)
{
    static $out;
    if (isset($out[$stylefile_abs])) {
        return $out[$stylefile_abs];
    }
    $new = array();
    $style = file_get_contents($stylefile_abs);
    /* die Werte entfernen */
    $style = preg_replace('#\{[^}]*\}#', '', $style);
    /* alle Klassen suchen */
    /* space / punkt / hoechstens 10 alphanumerische Zeichen inkl. dem Bindestrich / space*/
    preg_match_all('#\s\.([a-zA-Z0-9-]{1,10})\s#', $style, $matches);
    $style_list = array_unique($matches[1]);
    natcasesort ($style_list);
    // array_unshift  ( $style_list  , '- Normal -' );
    $new[''] = '- Normal -';
    foreach ($style_list as $value) {
        $new[$value] = $value;
    }
    $out[$stylefile_abs] = $new;
    return $new;
}

/* Die Liste der relevanten pragmaMx Module */
function mxSpawGetAvailableModuleData()
{
    static $modules;
    if (isset($modules)) {
        return $modules;
    }
    global $prefix;
    $modules = array('News' => array('wygid' => 'news', 'title' => _SUBMISSIONS),
        'Content' => array('wygid' => 'content', 'title' => _CONTENT),
        'Sections' => array('wygid' => 'sections', 'title' => _SECTIONS),
        'Downloads' => array('wygid' => 'dolo', 'title' => _DOWNLOADS),
        'Web_Links' => array('wygid' => 'linx', 'title' => _WEBLINKS),
        'Reviews' => array('wygid' => 'review', 'title' => _WREVIEWS),
        'Encyclopedia' => array('wygid' => 'enzy', 'title' => _ENCYCLOPEDIA),
        'FAQ' => array('wygid' => 'faq', 'title' => _FAQ),
        'xx_ephem' => array('wygid' => 'ephem', 'title' => _EPHEMERIDS),
        'xx_comments' => array('wygid' => 'comments', 'title' => _COMMENTS),
        'xx_topbox' => array('wygid' => 'topbox', 'title' => _BLK_MESSAGE),
        );
    $in = implode("','", array_keys($modules));
    $GLOBALS['mxSkipSqlDetect'] = true;
    $result = sql_query("SELECT title, custom_title, active FROM " . $prefix . "_modules WHERE title IN ('" . $in . "')");
    while ($row = sql_fetch_assoc($result)) {
        if (html_entity_decode($modules[$row['title']]['title']) != html_entity_decode($row['custom_title'])) {
            $modules[$row['title']]['title'] .= ' <span class="tiny">(' . $row['custom_title'] . ')</span>';
        }
    }
    return $modules;
}

/* Die Liste der pragmaMx Usergruppen */
function mxSpawGetUsergroupsList()
{
    static $groups;
    if (isset($groups)) {
        return $groups;
    }
    global $prefix;
    $GLOBALS['mxSkipSqlDetect'] = true;
    $result = sql_query("SELECT access_id, access_title FROM " . $prefix . "_groups_access ORDER BY access_title");
    $groups[$GLOBALS['default_group']] = MX_FIRSTGROUPNAME;
    while ($row = sql_fetch_assoc($result)) {
        $groups[$row['access_id']] = $row['access_title'];
    }
    return $groups;
}

/* guggen, ob der User so doof ist und benutzt den bescheuerten Internetexplorer... */
function mxSpawIsDrecksIE()
{
    // stripos() ist besser, aber erst ab php5 verfuegbar
    // return stripos($_SERVER['HTTP_USER_AGENT'], 'MSIE') !== false;
    return stristr($_SERVER['HTTP_USER_AGENT'], 'MSIE') !== false;
}

/* Check, ob der aktuelle User, bestimte Dateitypen hochladen darf */
function mxSpawFiletypeAllowed($filetype)
{
    if (in_array('any', $GLOBALS['my_wyscnf']['filetype'])) {
        return true;
    }
    if (is_array($filetype)) {
        foreach ($filetype as $value) {
            if (in_array($value, $GLOBALS['my_wyscnf']['filetype'])) {
                return true;
            }
        }
    } else {
        if (in_array($filetype, $GLOBALS['my_wyscnf']['filetype'])) {
            return true;
        }
    }
    return false;
}

?>