/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 1.1.2.4 $
 * $Author: tora60 $
 * $Date: 2008/01/09 02:42:35 $
 */

// Variable mit Browserkennung initialisieren
var pageBreakAgent = '';

function SpawPGpragmamx()
{
}

// Beim Umschalten des Bearbeitungsmodus koennen z.B. versch. 
// Platzhalter sichtbar gemacht werden
SpawPGpragmamx.beforemodeswitch = function(editor, event)
{
    var page = editor.getActivePage();
    if (page.editing_mode == "html") {
        var pta = editor.getPageInput(page.name);
        pta.value = SpawPGpragmamx.transform_pagebreak(pta.value, 'design');        
    } else if(page.editing_mode == "design") {
        var pdoc = editor.getPageDoc(page.name);
        pdoc.body.innerHTML = SpawPGpragmamx.transform_pagebreak(pdoc.body.innerHTML, 'html');        
    }
}

// Verarbeitung beim ersten Initialisieren des jeweiligen Editors
SpawPGpragmamx.init = function(editor, event)
{
    var pdoc = editor.getActivePageDoc();
    result = pdoc.body.innerHTML;
    // unnoetige br-Tags entfernen
    result = SpawPGpragmamx.clean_html_breaks(result);
    if (result) {
        // pagebreak korrigieren
        result = SpawPGpragmamx.transform_pagebreak(result, 'design');        
    }
    pdoc.body.innerHTML = result;
}

// Korrekturen beim ermitteln des Editor-Inhaltes
SpawPGpragmamx.gethtml = function(editor, event)
{
    var page = editor.getActivePage();
    var pdoc = editor.getPageDoc(page.name);
    pdoc.body.innerHTML = SpawPGpragmamx.clean_html_breaks(pdoc.body.innerHTML);        

    // gethtml wird auch beim Submit aufgerufen, deshalb hier den pagebreak korrigieren
    if(page.editing_mode == "design") {
        pdoc.body.innerHTML = SpawPGpragmamx.transform_pagebreak(pdoc.body.innerHTML, 'html');        
    }
}


// pagebreak fuer versch. Module, z.B. Content, korrigieren
// Das bewirkt, dass im HTML-Modus, dieser Platzhalter im Klartext angezeigt wird
// <!--pagebreak-->
SpawPGpragmamx.transform_pagebreak = function(html, mode)
{
    if (mode == 'design') {
        return html.replace(/&lt;!--pagebreak--&gt;|<!--pagebreak-->/g, '&lt;!--pagebreak--&gt;');
    } 
    if (mode == 'html') {
        return html.replace(/\s*(&lt;!--pagebreak--&gt;|<!--pagebreak-->)\s*/g, "\n\n\n<!--pagebreak-->\n\n");
    }
}

// fix unnoetiger <br> Tag in leeren Feldern?
// unnoetige br-Tags entfernen
SpawPGpragmamx.clean_html_breaks = function(html)
{
    /* unnoetigen Parameter in <br /> entfernen */
    html = html.replace(/<br\s*type="_moz"/g, '<br');
    /* zuerst leerezeichen drumrum entfernen */
    html = html.replace(/^\s+/g, '').replace(/\s+$/g, ''); 
    /* dann die Tags entfernen */
    html = html.replace(/(\s*<br\s*\/*>)+$/g, '');
    /* raus damit */
    return html;
}

// Klick auf den Pagebreakbutton in der Toolbar
SpawPGpragmamx.PagebreakClick = function(editor, tbi, sender) 
{ 
    if (tbi.is_enabled) { 
        
        if (editor.isInDesignMode()) {
            // im Designmode

            // Internetexplorer 7.x
            if (pageBreakAgent == 'ie') {
                // nur wenn kein Text selektiert ist
                var cur_node = editor.getNodeAtSelection();
                if (!cur_node.innerHTML) {
                    var tmp_node = cur_node;
                    // keine Ahnung was der Drecksbrowser da mit den Spans macht, aber so geht es...
                    tmp_node.innerHTML = tmp_node.innerHTML + '\<\/span>&lt;!--pagebreak--&gt;\<span>';
                    editor.insertNodeAtSelection(tmp_node);
                }
            } else {
                // gecko etc.
                // createElement, umstaendlich aber noetig in der Funktion insertHtmlAtSelection()
                var pdoc = editor.getActivePageDoc(); 
                var cur_node = editor.getNodeAtSelection();
                var tmp_node = pdoc.createElement("SPAN");
                tmp_node.appendChild(cur_node);
                // nur wenn kein Text selektiert ist
                if (!tmp_node.innerHTML) {
                    var what = '&lt;!--pagebreak--&gt;';
                    editor.insertHtmlAtSelection(what);
                }
            }
            editor.getPageIframe(editor.getActivePage().name).focus();
        } else {
            // im HTML-Modus ist das wie bei bbCode...
            var what = '<!--pagebreak-->';
            var textbox = editor.getPageInput(editor.getActivePage().name);
            // Unclassified NewsBoard
            // Copyright 2003-4 by Yves Goergen
            // Homepage: http://newsboard.unclassified.de
            if (textbox.createTextRange)        {
                textbox.focus();
                document.selection.createRange().duplicate().text = what;
                textbox.focus();
            } else if (textbox.selectionStart >= 0)        {
                // Mozilla without selected text
                var start_selection = textbox.selectionStart;
                var scroll_top = textbox.scrollTop;
                var scroll_left = textbox.scrollLeft;

                // fetch everything from start of text area to selection start
                var start = textbox.value.substring(0, start_selection);
                // fetch everything from end of selection to end of text area
                var end = textbox.value.substring(start_selection, textbox.textLength);

                textbox.value = start + what + end;

                textbox.selectionStart = textbox.selectionEnd = start_selection + what.length;
                textbox.focus();
                textbox.scrollTop = scroll_top;
                textbox.scrollLeft = scroll_left;
            } else        {
                textbox.value += what;
                textbox.focus();
            }
            // Unclassified NewsBoard ENDE
        }
    } 
} 

// guggen ob Pagebreak aktiv ist > immer true
// der Button wird aber je nach Modul in der toolbar direkt ein u. ausgeschaltet
SpawPGpragmamx.isPagebreakEnabled = function(editor, tbi) 
{ 
  return true; // editor.isInDesignMode(); 
} 

// Eventhandler zuordnen
SpawEngine.addEventHandler("spawinit", "SpawPGpragmamx.init");
SpawEngine.addEventHandler("spawbeforemodeswitch", "SpawPGpragmamx.beforemodeswitch");
SpawEngine.addEventHandler("spawgethtml", "SpawPGpragmamx.gethtml");
